import classNames from 'classnames';
import dayjs, { Dayjs } from 'dayjs';
import React, { CSSProperties, useMemo } from 'react';
import { memo } from 'react';
import FileIcon from '../FileIcon';
import ATooltip from '../Tooltip';
import styles from './FileStatus.less';
import { CloseCircleFilled, CheckCircleFilled } from '@ant-design/icons';
import { Progress, Button, ButtonProps } from 'antd';
import { NotificationType, TaskStatus, TaskType } from '@/services/API.helper';
import { DATA } from '@/services/API';
import { identity } from 'lodash';
import { useCallback } from 'react';
import { fetchLocalApi } from '@/utils/request';
import system from '@/services/system';

interface FileStatusProps {
  className?: string;
  style?: CSSProperties;
  data: DATA.SocketFileMsg;
}

export default function FileStatus(props: FileStatusProps) {
  const { className, style, data, ...restProps } = props;
  const filePath = useMemo(() => {
    return [data.projName, data.nodeName, data.relativePath]
      .filter(identity)
      .join(' / ');
  }, [data]);
  const relationModifier = useMemo(() => {
    return data.modifyName
      ? `${data.modifyName}同步`
      : data.createUserName
      ? `${data.createUserName}同步`
      : '';
  }, [data]);

  const redoTask = useCallback(() => {
    fetchLocalApi('restartTask', { taskIds: data.taskId });
  }, [data.taskId]);

  const downloadFile = useCallback(() => {
    fetchLocalApi('downloadFileFromMsg', { fileObject: data.originData });
  }, [data.taskId]);

  const checkFile = useCallback(() => {
    system.openFilePosition(data);
  }, [data]);

  return (
    <div className={classNames(styles.fileStatus, className)} style={style}>
      <div className={styles.left}>
        <FileIcon className={styles.icon} extension={data.extension} />
        <div className={styles.content}>
          <ATooltip placement="top" title={data.archName}>
            <div className={styles.fileName}>{data.archName}</div>
          </ATooltip>
          <div className={styles.subContent}>
            <ATooltip placement="bottom" title={filePath}>
              <div className={styles.filePath}>{filePath}</div>
            </ATooltip>
            <div className={styles.modifyInfo}>{relationModifier}</div>
          </div>
        </div>
      </div>
      <div className={styles.mid}>
        {data.notifyType ? (
          <span>{data.notifyMessage}</span>
        ) : (
          <LoadDesc
            type={data.taskType}
            loadingState={data.taskSyncStatus}
            progress={data.taskSyncProgress}
            {...restProps}
          />
        )}
        {data.taskSyncStatus === TaskStatus.FINISH || data.notifyType ? (
          <Time time={data.taskCreateDate} />
        ) : null}
      </div>
      <div className={styles.right}>
        {/* 查看1: 已下载 文件打开文件夹 */}
        {data.taskSyncStatus === TaskStatus.FINISH ? (
          <ActionButton onClick={checkFile}>查看</ActionButton>
        ) : null}
        {/* 重新下载: 下载失败时出现 */}
        {/* 重新上传: 上传失败时出现 */}
        {data.taskSyncStatus === TaskStatus.FAILED ? (
          <ActionButton onClick={redoTask}>
            {data.taskType === TaskType.DOWNLOAD ? '重新下载' : '重新上传'}
          </ActionButton>
        ) : null}
        {data.notifyType && data.notifyType === NotificationType.SYNC ? (
          <ActionButton onClick={downloadFile}>下载</ActionButton>
        ) : null}
        {/* <Button type="link" className={styles.button} size="small">暂停</Button> */}
        {/* <Button type="link" className={styles.button} size="small">取消</Button> */}
      </div>
    </div>
  );
}

function ActionButton({ onClick, children }: ButtonProps) {
  return (
    <Button
      type="link"
      className={styles.button}
      size="small"
      onClick={onClick}
    >
      {children}
    </Button>
  );
}
interface TimeProps {
  time: Dayjs;
}
const Time = memo(({ time }: TimeProps) => {
  const now = dayjs();
  const isToday = time.isSame(now, 'day');
  const isSameYear = time.isSame(now, 'year');
  const formatStr = useMemo(() => {
    if (isToday) return 'HH:mm:ss';
    if (isSameYear) return 'MM/DD HH:mm:ss';
    return 'YYYY/MM/DD HH:mm:ss';
  }, [isToday, isSameYear]);
  return <span className={styles.time}>{time.format(formatStr)}</span>;
});

interface LoadDescProps {
  type: TaskType;
  loadingState: TaskStatus;
  result?: 'success' | 'fail';
  progress?: number;
}

function LoadDesc(props: LoadDescProps) {
  const { result, type, loadingState, progress } = props;
  const keywords = type === 'UPLOAD' ? '上传' : '下载';
  const [resultText, icon] = useMemo(() => {
    if (loadingState === TaskStatus.FINISH) {
      return [
        `${keywords}成功!`,
        <CheckCircleFilled
          className={classNames(styles.stateIcon, styles.success)}
        />,
      ];
    }
    if (loadingState === TaskStatus.FAILED) {
      return [
        `${keywords}失败!`,
        <CloseCircleFilled
          className={classNames(styles.stateIcon, styles.error)}
        />,
      ];
    }
    if (loadingState === TaskStatus.WAITING) {
      return [`等待${keywords}`];
    }
    return ['', null];
  }, [result, keywords]);
  return (
    <span className={styles.loadDesc}>
      {loadingState === TaskStatus.SYNCING ? (
        <Progress percent={progress} format={() => `${keywords}中...`} />
      ) : (
        <>
          {icon}
          {resultText}
        </>
      )}
    </span>
  );
}
